function [aH,bH,cH,ar,br,cr,h0,h1] = LogLin(S)

% Description:
% Campbell-Viceira log-linear approximation of Bansal and Yaron's model for
% values of EIS different to one. 
%
% Inputs:
% S = Parameter structure
%
% Outputs:
% [aH, bH] = Coefficients for the H approximation: H = exp(aH + bH*mu + cH*v)
% [ar, br] = Coefficients for the approximated risk-free rate: r = ar +  br*mu + cr*v
%
% Written by:
% Juan Carlos Parra-Alvarez
% Aarhus University and CREATES
% February, 2018

%% Model parameters
beta      = S.beta;
gamma     = S.gamma;
psi       = S.psi;
mu_bar    = S.mu_bar;
v_bar     = S.v_bar;
kappa_mu  = S.kappa_mu;
kappa_v   = S.kappa_v;
nu_mu     = S.nu_mu;
nu_v      = S.nu_v;

if v_bar<0
  error('Long-run volatility negative')
end

% Some useful transformations
theta = (1-gamma)/(1-psi^(-1));

% -------------------------------------------------------
% Price-consumption ratio
% -------------------------------------------------------
maxit = S.maxitLogLin;
crit  = S.crit;
h1_0  = beta;
h1    = h1_0;

for j = 1:maxit
    HH  = h1;
    h0 = HH*(1-log(HH));
    bH = ((1-gamma)/psi)*(1/(kappa_mu+HH));
    cp = (psi*(HH+kappa_v))/(nu_v^2*psi^2) + sqrt(((psi*(HH+kappa_v))/(nu_v^2*psi^2))^2 - (nu_mu^2*psi^2*bH^2 - gamma*(1-gamma))/(nu_v^2*psi^2));
    cn = (psi*(HH+kappa_v))/(nu_v^2*psi^2) - sqrt(((psi*(HH+kappa_v))/(nu_v^2*psi^2))^2 - (nu_mu^2*psi^2*bH^2 - gamma*(1-gamma))/(nu_v^2*psi^2));
    cH = cn;
    aH = (1/(psi*HH))*(theta*h0 + psi*theta*HH*log(beta)-theta*beta + psi*kappa_mu*mu_bar*bH + psi*kappa_v*v_bar*cH);
    HH = (beta^psi)*exp(((1-psi)/(1-gamma))*(aH + bH*mu_bar + cH*v_bar));

    h1_change = HH - h1;
    h1 = HH;
 
    distC(j) = max(abs(h1_change));
    
    if distC(j)<crit
        disp(' ')
        fprintf(' H(lambda) function converged at iteration = %5.0f\n', j);
        disp(' ')
        break
    end
    
    if j>=maxit
        disp(' ')
        fprintf('-------------------------------------------------- \n');
        fprintf(' H(lambda) function did not converge \n');
        disp(' ')
        fprintf(' Maximum number of iterations reached \n');
        disp(' ')
        fprintf(' Current norm: = %5.8f\n', distC(j));
        fprintf('-------------------------------------------------- \n');
        disp(' ')
        return
    end
end

%H = exp(aH + bH*mu + cH*v); % Approximated h(lambda)

% -------------------------------------------------------
% Risk-free rate
% -------------------------------------------------------

ar = h0 + h1*psi*log(beta) + h1*((1-psi)/(1-gamma))*aH - ((1-psi)/(1-gamma))*bH*kappa_mu*mu_bar  - ((1-psi)/(1-gamma))*cH*kappa_v*v_bar;
br = 1 + ((1-psi)/(1-gamma))*(kappa_mu + h1)*bH; 
cr = ((1-psi)/(1-gamma))*cH*(kappa_v + h1) - gamma - 0.5*((1-psi)/(1-gamma))^2*((1+psi-2*gamma*psi)/(1-gamma))*(bH^2*nu_mu^2 + cH^2*nu_v^2);
%r    = ar + br*mu + cr*v;   

% End-of-file
