%% =======================================================================%
%  Peso Problems in the Estimation of the C-CAPM                          %
%  Simualtion of the long-run risk model                                  %    
%  Quantitative Economics (forthcoming)                                   %
%                                                                         %                                                                         %
% ----------------------------------------------------------------------- %
%  NOTES: 
% ----------------------------------------------------------------------- %
%  Main_LRR.m requires the following complementary codes available in the 
%  folder:
%
%  1. Paramstructbase.m : Defines parameter values
%
%  2. Loglin.m : Computes the coefficients of the log-linear approximation
%
%  3. sim_state.m : Simulates paths for the state variables (modified 
%  (versions of those originally written by Jessica Wacther 
%  (2013, Journal of Finance))
%
%  4. sim_c.m : Simulates paths for the consumption process (modified 
%  (versions of those originally written by Jessica Wacther 
%  (2013, Journal of Finance))
%
%  5. simret.m, annobs.m, and compret.m : Simulate returns, compute annual
%  observations from simulated paths, and compute corresponding compounded 
%  returns (codes are modified versions of those originally written by 
%  Jessica Wacther (2013, Journal of Finance). 
%
% ----------------------------------------------------------------------- %
% (c) Juan Carlos Parra-Alvarez, Olaf Posch and Andreas Schrimpf            
% Date March, 2021.
%
function main_LRR(startdate)
% using a function to keep current workspace; uncomment to use separately

%% =======================================================================%
%                             Housekeeping                                %
%-------------------------------------------------------------------------%
% clear
% clc
% close all
   
%-------------------------------------------------------------------------%
%                            Activate plots                               %
%-------------------------------------------------------------------------%
run_plot = 0;

%-------------------------------------------------------------------------%
%                              Save output                                %
%-------------------------------------------------------------------------%
save_output = 1;

%-------------------------------------------------------------------------%
%                       Load structural parameter                         %
%-------------------------------------------------------------------------%
% Structure with model parameters
S  = paramstructbase;

beta      = S.beta;
gamma     = S.gamma;
psi       = S.psi;
mu_bar    = S.mu_bar;
v_bar     = S.v_bar;
kappa_mu  = S.kappa_mu;
kappa_v   = S.kappa_v;
nu_mu     = S.nu_mu;
nu_v      = S.nu_v;
theta     = (1-gamma)/(1-psi^(-1));

if v_bar<0
  error('Long-run volatility negative')
end

if gamma - 1/psi > 0
    disp(' ')
    fprintf('------------------------------------------------------------\n')
    fprintf(' Early resolution of uncertainty  \n');
    fprintf('------------------------------------------------------------\n')
    disp(' ')
elseif gamma - 1/psi < 0
    disp(' ')
    fprintf('------------------------------------------------------------\n')
    fprintf('  Late resolution of uncertainty  \n');
    fprintf('------------------------------------------------------------\n')
    disp(' ')
elseif gamma - 1/psi == 0
    disp(' ')
    fprintf('------------------------------------------------------------\n')
    fprintf('  Indifference about resolution of uncertainty  \n');
    fprintf('------------------------------------------------------------\n')
    disp(' ')
end

%-------------------------------------------------------------------------%
%                         Simulation parameters                           %
%-------------------------------------------------------------------------%
Tsim = S.Tsim;
freq = S.freq;
prec = S.prec;

% Random number generator seed
randn('state',305);
rand('state',200);

%=========================================================================%
%                 Policy function approximation step                      %
%=========================================================================%
%                      Definition of state space                          %
%-------------------------------------------------------------------------%
n       = S.n; 
mu_min  = norminv(0.025,mu_bar,sqrt(nu_mu^2*v_bar/(2 * kappa_mu)));
mu_max  = norminv(0.975,mu_bar,sqrt(nu_mu^2*v_bar/(2 * kappa_mu)));
v_min   = gaminv(0.025,2*kappa_v*v_bar/nu_v^2,nu_v^2/(2 * kappa_v));
v_max   = gaminv(0.975,2*kappa_v*v_bar/nu_v^2,nu_v^2/(2 * kappa_v));
s_min   = [mu_min,v_min];
s_max   = [mu_max,v_max];
fspace  = fundefn('cheb',[n n],s_min,s_max);
d       = fspace.d; 
scoord  = funnode(fspace);
snodes  = gridmake(scoord);
mu      = snodes(:,1);
v       = snodes(:,2);
[muX,vX]= meshgrid(scoord{1},scoord{2});

%% =======================================================================%
%                       Log-linear approximation                          %
%-------------------------------------------------------------------------%
disp(' ')
fprintf('------------------------------------------------------------- \n');
fprintf(' Log-linear approximations: \n')

[aH,bH,cH,ar,br,cr,h0,h1] = LogLin(S);
H_lin                     = exp(aH + snodes*[bH; cH]);
rf_lin                    = ar + snodes*[br; cr];
fprintf('------------------------------------------------------------- \n');
disp(' ');
disp('Checking for irrational solution of H ...');
disp(' ');
if isreal(H_lin) ~= 1 
    warning('Imaginary roots: calculations stopped');
    disp(' ');
    disp('Verify calibration');
    return;
else
    disp('No imaginary roots in the approximation')
end
fprintf('------------------------------------------------------------- \n');

% Reshaping log-lin vectors into matrices
H_lin   = reshape(H_lin,[n n]);
rf_lin  = reshape(rf_lin,[n n]);
c_w_lin = h0 + h1*psi*log(beta)+h1*((1-psi)/(1-gamma))*log(H_lin);

% Plots of log-linear and global approximations to H, rf and G.
if run_plot == 1    
    figure(1)
    subplot(2,1,1)
    surf(muX,vX,c_w_lin)
    title('Consumption-Wealth ratio','interpreter','latex','fontsize',14)
    ylabel('$\vartheta$','interpreter','latex','fontsize',14)
    xlabel('$\mu$','interpreter','latex','fontsize',14)
    axis([-inf inf -inf inf -inf inf])
    set(gca,'fontsize',14)
    set(gcf,'color','w')
    
    subplot(2,1,2)
    surf(muX,vX,rf_lin)
    title('Risk-free rate','interpreter','latex','fontsize',14)
    ylabel('$\vartheta$','interpreter','latex','fontsize',14)
    xlabel('$\mu$','interpreter','latex','fontsize',14)
    axis([-inf inf -inf inf -inf inf])
    set(gca,'fontsize',14)
    set(gcf,'color','w')
end

%% =======================================================================%
%                            Simulation step                              %
%=========================================================================%
% Allocation of memory
delc_ann_j      = zeros(Tsim,S.MCsim);
reqlin_ann_j    = zeros(Tsim,S.MCsim);
rbondlin_ann_j  = zeros(Tsim,S.MCsim);
rflin_ann_j     = zeros(Tsim,S.MCsim);

loopbar = waitbar(0,'Sampling');
for j = 1:S.MCsim
    waitbar(j/S.MCsim)
    %-------------------------------------------------------------------------%
    %    Expected consumption growth and Volatility of consumption growth     %
    %-------------------------------------------------------------------------%
    [mut, vt]  = sim_state(mu_bar, v_bar,kappa_mu, kappa_v,nu_mu, nu_v,Tsim,freq,prec,0);
    st{1}      = mut;
    st{2}      = vt;
    
    %-------------------------------------------------------------------------%
    %                      Consumption and dividends                          %
    %-------------------------------------------------------------------------%
    delc = sim_c(mut,vt,Tsim,freq,prec);
    delc_mon_j(:,j) = delc;
    
    % Create annual observations for growth rates of consumption and dividends
    delc_ann           = compret(delc(1:end-1,1),freq*prec);
    delc_ann_j(:,j)    = delc_ann;
    
    % Compute autocorrelations
    SimOut.Dc_AC(:,j)      = autocorr(delc_ann,10);
    SimOut.Dc_VR2_j(j,:)   = 1+SimOut.Dc_AC(2,j);
    SimOut.Dc_VR5_j(j,:)   = 1+2*[1-1/5,1-2/5,1-3/5,1-4/5]*[SimOut.Dc_AC(2,j) SimOut.Dc_AC(3,j) SimOut.Dc_AC(4,j) SimOut.Dc_AC(5,j)]';
    SimOut.Dc_VR10_j(j,:)  = 1+2*[1-1/10,1-2/10,1-3/10,1-4/10,1-5/10,1-6/10,1-7/10,1-8/10,1-9/10]*[SimOut.Dc_AC(2,j) SimOut.Dc_AC(3,j) SimOut.Dc_AC(4,j) SimOut.Dc_AC(5,j) SimOut.Dc_AC(6,j) SimOut.Dc_AC(7,j) SimOut.Dc_AC(8,j) SimOut.Dc_AC(9,j) SimOut.Dc_AC(10,j)]';
    
    %=========================================================================%
    %                     Log-linearized simulations                          %
    %-------------------------------------------------------------------------%
    %                     Interpolation of H(mu,v)                            %
    %-------------------------------------------------------------------------%
    nsim     = Tsim*freq*prec + 1;
    Hlin_t   = zeros(nsim,1);
    Hmulin_t = zeros(nsim,1);
    Hmu2lin_t= zeros(nsim,1);
    Hvlin_t  = zeros(nsim,1);
    Hv2lin_t  = zeros(nsim,1);
    
    for i = 1:nsim
        Hlin_t(i,1)    = exp(aH+bH*mut(i)+cH*vt(i));
        Hmulin_t(i,1)  = bH*Hlin_t(i,1);
        Hmu2lin_t(i,1) = bH^2*Hlin_t(i,1);
        Hvlin_t(i,1)   = cH*Hlin_t(i,1);
        Hv2lin_t(i,1)  = cH^2*Hlin_t(i,1);
    end
    
    %-------------------------------------------------------------------------%
    %                       Consumption-wealth ratio                          %
    %-------------------------------------------------------------------------%
    c_w_lin_t = (beta^psi)*Hlin_t.^((1-psi)/(1-gamma));
    cw_ratio_mom(:,j) = c_w_lin_t;
    
    % annual
    cw_lin_t_ann = annobs(c_w_lin_t,freq*prec);
    cw_ratio_yoy(:,j) = cw_lin_t_ann;
    
    %quarterly
    step0 = reshape(c_w_lin_t(2:end),prec,Tsim);
    for i = 1:Tsim
        step = step0(:,i);
        step = reshape(step,3,4);
        step = step(end,:);
        step1(:,i) = step';
    end
    step2 = [c_w_lin_t(1);reshape(step1,4*Tsim,1)];
    cw_ratio_qoq(:,j) = step2;
        
    %-------------------------------------------------------------------------%
    %                      Price-dividend ratio = 1/(c/w)                     %                          %
    %-------------------------------------------------------------------------%
    % Create annual observations for the log_pd ratio
    pdlin_ann            = annobs(1./c_w_lin_t,freq*prec);
    log_pdlin_ann        = log(pdlin_ann);
    log_pdlin_ann_j(:,j) = log_pdlin_ann;
    log_pdlin_AC_j(:,j)  = autocorr(log_pdlin_ann,1);
    
    %-------------------------------------------------------------------------%
    %                             Risk-Free rate                              %
    %-------------------------------------------------------------------------%    
    rflin_t = c_w_lin_t + mut - ((1-psi)/(1-gamma))*((Hmulin_t./Hlin_t)*kappa_mu.*(mu_bar - mut) + (Hvlin_t./Hlin_t)*kappa_v.*(v_bar-vt)) ...
        - 0.5*((1-psi)/(1-gamma))*((-(1-psi)/(1-gamma)-1)*(Hmulin_t./Hlin_t).^2 + Hmu2lin_t./Hlin_t)*nu_mu^2.*vt ...
        - 0.5*((1-psi)/(1-gamma))*((-(1-psi)/(1-gamma)-1)*(Hvlin_t./Hlin_t).^2 + Hv2lin_t./Hlin_t)*nu_v^2.*vt ...
        - gamma*(vt+((1-psi)/(1-gamma))^2*(Hmulin_t./Hlin_t).^2*nu_mu^2.*vt + ((1-psi)/(1-gamma))^2*(Hvlin_t./Hlin_t).^2*nu_v^2.*vt) ...
        - ((1-psi)/(1-gamma))*(nu_mu^2*(Hmulin_t./Hlin_t).^2 + nu_v^2*(Hvlin_t./Hlin_t).^2).*vt;
    
    %-------------------------------------------------------------------------%
    %                        Simulate gross returns                           %
    %-------------------------------------------------------------------------%
    [rbondlin, rclin, rc2lin] = simret(rflin_t, c_w_lin_t, delc(1:end-1,1),freq*prec);
    rbondlin_mon_j(:,j) = rbondlin;
    rclin_mon_j(:,j)    = rclin;
    rclin2_mon_j(:,j)   = rc2lin;
    
    % Create annual observations for equity returns and government bonds returns
    rbondlin_ann        = compret(log(rbondlin),freq*prec);
    rbondlin_ann        = exp(rbondlin_ann);
    rbondlin_ann_j(:,j) = rbondlin_ann;
    rflin_ann           = compret(log(1+rflin_t(1:end-1,:)/(freq*prec)),freq*prec);
    rflin_ann           = exp(rflin_ann);
    rflin_ann_j(:,j)    = rflin_ann; 
    rclin_ann           = compret(log(rclin),freq*prec);
    rclin_ann           = exp(rclin_ann);
    rclin_ann_j(:,j)    = rclin_ann;
end 
close(loopbar);
disp(['Sampling:',' done.'])

% Annualized growth rates of consumption
dc_yoy_ann = delc_ann_j;
dc_mom_ann = delc_mon_j(1:end-1,:).*12;
dc_qoq_ann = reshape(delc_mon_j(1:end-1,:),prec,Tsim,S.MCsim);
dc_qoq_ann = dc_qoq_ann(3:3:end,:,:);
dc_qoq_ann = reshape(dc_qoq_ann,4*Tsim,S.MCsim);
dc_qoq_ann = dc_qoq_ann.*12;

% Non-annualized growth rates of consumption
dc_mom    = delc_mon_j(1:end-1,:);
dc_qoq_v0 = reshape(delc_mon_j(1:end-1,:),prec,Tsim,S.MCsim);
for i = 1:Tsim
    for j = 1:S.MCsim
        step = dc_qoq_v0(:,i,j);
        step = reshape(step,3,4);
        step = sum(step);
        step = reshape(step,4,1);
        dc_qoq(:,i,j) = step;
    end
end
dc_qoq = reshape(dc_qoq,4*Tsim,S.MCsim);
dc_qoq = dc_qoq;

% Level of consumption rates
c_yoy_ann = cumprod(1+dc_yoy_ann);
c_mom_ann = cumprod(1+dc_mom_ann);
c_qoq_ann = cumprod(1+dc_qoq_ann);
c_mom     = cumprod(1+dc_mom);
c_qoq     = cumprod(1+dc_qoq);

% Consumption-to-wealth ratio
cw_mom = cw_ratio_mom(2:end,:);
cw_yoy = cw_ratio_yoy(2:end,:);
cw_qoq = cw_ratio_qoq(2:end,:);

% Price of risky asset
Pd_mom = c_mom./cw_mom;
Pd_yoy = c_yoy_ann./cw_yoy;
Pd_qoq = c_qoq./cw_qoq;

% Annualized risk-free return
rbond_yoy_ann  = rbondlin_ann_j;
rbond_mom_ann  = rbondlin_mon_j.^12;
rbondlin_qrt_j = reshape(rbondlin_mon_j,prec,Tsim,S.MCsim);
rbondlin_qrt_j = rbondlin_qrt_j(3:3:end,:,:);
rbondlin_qrt_j = reshape(rbondlin_qrt_j,4*Tsim,S.MCsim);
rbond_qoq_ann  = rbondlin_qrt_j.^12;

rbond_mom = rbondlin_mon_j;
rbond_qoq1= reshape(log(rbondlin_mon_j),12,Tsim,S.MCsim);
for i = 1:S.MCsim
    for j = 1:Tsim
        step = reshape(rbond_qoq1(:,j,i),3,4);
        step = sum(step);
        step = exp(step);
        rbond_qoq2(:,j) = step';
    end    
    rbond_qoq(:,:,i) = rbond_qoq2;
end
rbond_qoq = reshape(rbond_qoq,4*Tsim,S.MCsim);

% Annualized return on consumption claim
rc_yoy_ann     = rclin_ann_j;
rc_mom_ann     = rclin_mon_j.^12;
rclin_qrt_j    = reshape(rclin_mon_j,prec,Tsim,S.MCsim);
rclin_qrt_j    = rclin_qrt_j(3:3:end,:,:);
rclin_qrt_j    = reshape(rclin_qrt_j,4*Tsim,S.MCsim);
rc_qoq_ann     = rclin_qrt_j.^12;

rc_mom = rclin_mon_j;
rc_qoq1= reshape(log(rclin_mon_j),12,Tsim,S.MCsim);
for i = 1:S.MCsim
    for j = 1:Tsim
        step = reshape(rc_qoq1(:,j,i),3,4);
        step = sum(step);
        step = exp(step);
        rc_qoq2(:,j) = step';
    end    
    rc_qoq(:,:,i) = rc_qoq2;
end
rc_qoq = reshape(rc_qoq,4*Tsim,S.MCsim);

if save_output == 1
    simversion ='v1';
%     startdate = date;
    cd ..
    save([cd '/Data/Draw_' startdate '/Samples/' 'LRR_Sim_',simversion,'.mat'],...
        'dc_yoy_ann', 'dc_mom_ann', 'dc_qoq_ann',...
        'dc_mom', 'dc_qoq', ...
        'c_yoy_ann', 'c_qoq_ann', 'c_mom_ann',...
        'c_qoq', 'c_mom',...
        'rbond_yoy_ann', 'rbond_qoq_ann' ,'rbond_mom_ann',...
        'rbond_qoq' ,'rbond_mom',...
        'rc_yoy_ann','rc_qoq_ann','rc_mom_ann',...
        'rc_qoq' ,'rc_mom',...
        'Pd_yoy','Pd_qoq','Pd_mom');
end



% End-of-file