%% =======================================================================%
%  Peso Problems in the Estimation of the C-CAPM                          %
%  Empirical C-CAPM estimates                                             %    
%  Quantitative Economics (forthcoming)                                   %
%
%  (0) Housekeeping                                                       %  
%  (1) Import data and select sample                                      %  
%  (2) Results                                                            %  
% ----------------------------------------------------------------------- %
%  NOTES: 
% ----------------------------------------------------------------------- %
%  Empirical_estimates.m requires the following complementary files       %
%  available in the folder:
%
%  1. USFF_data.xlsx: 
%  Data description: 
%  - Col 3 3-Month Treasury Bill [TB3MS], Quarterly, End of Period,
%  retrieved from FRED, Federal Reserve Bank of St. Louis; 
%  https://fred.stlouisfed.org/series/TB3MS.  
%  - Col 4 implied quarterly gross return (RF) on 3-Month T-Bill
%  - Cols 5-11 US quarterly returns on a broad stock market index
%  (CRSP value-weighted price index return), and 6 Fama-French portfolios;  
%  https://mba.tuck.dartmouth.edu/pages/faculty/ken.french/data_library.html
%  - Col 12 implied quarterly stock market gross return (MKT)
%  - Col 13 implied quarterly stock market excess return (MKTX)
%  - Col 14 real (chain-weighted) personal consumption expenditures on 
%  nondurable goods and services per capita at a quarterly frequency 
%  Source: US Bureau of Economic Analysis, Real personal consumption 
%  expenditures per capita: Nondurable goods [A796RX0Q048SBEA], FRED; 
%  https://fred.stlouisfed.org/series/A796RX0Q048SBEA. 
%  We use the timing convention that consumption takes place at 
%  the end of a period. 

%  2. Global_data.xls: 
%  Data description: 
%  - Excess returns (EXRET), market return (M), and risk-free return (RF)
%  obtained from Global Financial Data (1900-2008) for selected countries
%  including the United States (US), Canada (CAN), Germany (GER), 
%  Italy (ITA), Japan (JAP), and United Kingdom (UK); 
%  https://www.globalfinancialdata.com
%  - Consumption data from the Barro-Ursúa macroeconomic data set;
%  https://scholar.harvard.edu/barro/data_sets
%
%  implemented using Matlab R2021a with toolboxes optim
% ----------------------------------------------------------------------- %
% (c) Juan Carlos Parra-Alvarez, Olaf Posch and Andreas Schrimpf            
% Date July, 2021.
% ----------------------------------------------------------------------- %

%% =======================================================================%
% (0) Housekeeping                                                        %
%-------------------------------------------------------------------------%
clear
clc
close all

%% =======================================================================%
% (1) Import data
%-------------------------------------------------------------------------%
% Read data from file
A_Global = xlsread('Data/Global_data.xlsx',1);
Q_US     = xlsread('Data/USFF_data.xlsx',1);

%-------------------------------------------------------------------------%
% Select quarterly or annual data
%-------------------------------------------------------------------------%
% 1  annual data
% 2  quarterly data
data = 2;

%-------------------------------------------------------------------------%
% Select sample (annual data only)
%-------------------------------------------------------------------------%
% 1  Long sample (US from 1900 to 2007), 
% 2  Post WWII sample
sample = 2;           

%-------------------------------------------------------------------------%
% Select country (annual data only)
%-------------------------------------------------------------------------%
% 1  Australia 						
% 2  Belgium 						
% 3  Canada 						
% 4  Denmark 						
% 5  France 						
% 6  Germany 						
% 7  Italy						
% 8  Japan 						
% 9  Netherlands 						
% 10 Norway 						   
% 11 Spain 						
% 12 Sweden 						 
% 13 Switzerland 						
% 14 UK						
% 15 U.S.
ctry = 15;          

%-------------------------------------------------------------------------%
% Consumption timing assumption (annual data only)
%-------------------------------------------------------------------------%
EOP = 1;   % EOP:1 - end of period assumption, EOP:2 - begin of period 

%-------------------------------------------------------------------------%
% Choose sample length (annual data)
%-------------------------------------------------------------------------%
if sample==1 && data ==1
	start_obs=33;  %  33: 1900
	end_obs=141;   % 141: 2008
    if ctry==3
        start_obs=67;
    end
elseif sample==2 && data ==1
	start_obs=84;   %  84: 1951
	end_obs=141;    % 141: 2008
end
%-------------------------------------------------------------------------%
% Choose sample length (quarterly data)
%-------------------------------------------------------------------------%
if sample==1 && data ==2
    error('Please select annual data for longer sample.')
elseif sample==2 && data ==2
%   start_obs=84;   %  84: 1947Q2
    start_obs=102;  % 102: 1951Q4
%   start_obs=131;  % 131: 1959Q1
%   start_obs=195;  % 195: 1975Q1
%   end_obs=298;    % 298: 2000Q4
%	end_obs=306;    % 306: 2002Q4
  end_obs=362;    % 362: 2016Q4
end

% trim data
if data==1
    mat=A_Global(start_obs:end_obs,:);
elseif data==2
    mat=Q_US(start_obs:end_obs,:);
end

% select sample country for investigation for annual data
if data==1
    if     ctry==1;	ctrystr='AU';
        ctrymat=mat(:,2:7);
    elseif ctry==2;	ctrystr='BE';
        ctrymat=mat(:,8:14);
    elseif ctry==3;	ctrystr='CA';
        ctrymat=mat(:,14:19);
    elseif ctry==4;	ctrystr='DK';
        ctrymat=mat(:,20:25);
    elseif ctry==5;	ctrystr='FR';
        ctrymat=mat(:,26:31);
    elseif ctry==6;	ctrystr='DE';
        ctrymat=mat(:,32:37);
    elseif ctry==7;	ctrystr='IT';
        ctrymat=mat(:,38:43);
    elseif ctry==8; ctrystr='JP';
        ctrymat=mat(:,44:49);
    elseif ctry==9;	ctrystr='NL';
        ctrymat=mat(:,50:55);
    elseif ctry==10;ctrystr='NW';
        ctrymat=mat(:,56:61);
    elseif ctry==11;ctrystr='ES';
        ctrymat=mat(:,62:67);
    elseif ctry==12;ctrystr='SW';
        ctrymat=mat(:,68:73);
    elseif ctry==13;ctrystr='CH';
        ctrymat=mat(:,74:79);
    elseif ctry==14;ctrystr='UK';
        ctrymat=mat(:,80:85);
    elseif ctry==15;ctrystr='US';
        ctrymat=mat(:,86:91);
    end
elseif data==2
    ctrystr='US';
end
% select labels
if data==1
    data_label = mat(:,1);
elseif data==2
    data_label = mat(:,1:2);
end

% select consumption ratio
if data==1
    cg_data=ctrymat(:,2:3);     % consumption ratio
    if EOP==1
        cg=cg_data(:,1);        % consumption end-of-period
    elseif EOP==2
        cg=cg_data(:,2);        % consumption begin-of-period
    end
elseif data==2
    cg=mat(:,14);                % Nondurables
%   cg=mat(:,15);                % Total consumption
end

% select return data
if data==1
    RF = ctrymat(:,6)./100+1;   % gross risk-free return = 1 + period return
    MKT = ctrymat(:,5)./100+1;  % gross market return = 1 + rate of return
    MKTX = MKT-RF;
%     MKTX = log(MKT)-log(RF);  % alternatively use log returns
elseif data==2
    RF = mat(:,4);
    MKT=mat(:,12);
    MKTX=MKT-RF;
    FF = mat(:,6:11)./100+1;
end

% select moment conditions and GMM instruments
T=size(cg,1);
b = [.5;10];     % starting values

% select options for minimizer
algo='Levenberg-Marquardt';
options= optimset('Display','off','MaxIter', 50000, 'MaxFunEvals', 50000,'TolFun', sqrt(eps),'Algorithm',algo, 'Jacobian', 'off');

%% =======================================================================%
% (2) Results
%-------------------------------------------------------------------------%
% Table A.1
%-------------------------------------------------------------------------%
% specify frequency for calculation of conditional EE errors
if data==2
    freq=1/4;
else
    freq=1;
end

% fixed structural parameters (if needed)
nu_bar=-0.4;
nu=-.75;
lambda=0.017;
alpha=0.33;
lambda_bar=0;
mu_bar = 0.02;
sigma_bar = 0.01;
sigma = 0.005;
q = 0;
rho = 0.03;
gamma = 4;
% alpha1 = exp((1-exp(-alpha*nu*gamma))*lambda*freq)-1;
% alpha1 = exp((1-exp(-nu_bar*gamma))*lambda*freq)-1;
if freq == 1/4
    alpha1 = -0.01;
else
    alpha1 = -0.05;
end

% Two assets (risky and riskless)
y = [ones(T,1) ones(T,1)];   % matrix containing the population mean of moment conditions
X = [cg MKT MKTX];
Z = ones(T,1);                % vector of instruments

%-------------------------------------------------------------------------%
% GMM estimates
%-------------------------------------------------------------------------%
% GMM (model-free) equation
[sdfpar_gmm,fval_gmm,exitflag_gmm] = fminsearch(@gmm_2_func,b,options,y,X,Z,eye(2));  % Call minimizer
% GMM, conditional on no disastennnrs (endowment)
[sdfpar_cond_endow_gmm,fval_cond_endow_gmm,exitflag_cond_endow_gmm] = fminsearch(@gmm_cond_endow_func,b,options,y,X,Z,eye(2),freq,nu_bar,lambda);      % Endowment
% GMM, conditional on no disasters (production rho=rho_bar)
[sdfpar_cond_prod2_gmm,fval_cond_prod2_gmm,exitflag_cond_prod2_gmm] = fminsearch(@gmm_cond_prod2_func,b,options,y,X,Z,eye(2),freq,alpha,nu,lambda,mu_bar,sigma_bar,sigma);  % Production 
% GMM, Parker/Julliard (model-free)
[sdfpar_cond_euler_gmm,~,~] = fminsearch(@gmm_cond_euler_func,[0;0],options,y,X,Z,eye(2),freq,gamma,rho);  % (33)
[sdfpar_cond_PJfix_gmm,~,~] = fminsearch(@gmm_cond_PJfix_1_func,[0.1;1],options,y,X,Z,eye(2),freq,rho,alpha1);  % (34)

% fitted SDF and moment conditions
sdf_fitted_func=@(beta,gamma) beta*cg.^(-gamma);
euler1_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*RF-ones(T,1);
euler2_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*MKT-ones(T,1);
euler1_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*RF-ones(T,1)*exp((1-exp(-gamma*nu_bar))*lambda*freq);
euler2_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*MKT-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
euler1_cond_prod2_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*RF - ones(T,1)*exp((1-exp(-alpha*gamma*nu))*lambda*freq);
euler2_cond_prod2_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*MKT - ones(T,1);
euler1_cond_euler_func = @(alpha_1,alpha_2) exp(-rho*freq)*cg.^(-gamma).*RF-ones(T,1)-alpha_1;
euler2_cond_euler_func = @(alpha_1,alpha_2) exp(-rho*freq)*cg.^(-gamma).*MKT-ones(T,1)-alpha_2;

euler1_cond_PJfix_func = @(alpha0,gamma) cg.^(-gamma).*RF - ones(T,1)*(1+alpha1)/exp(-rho*freq);
euler2_cond_PJfix_func = @(alpha0,gamma) exp(-rho*freq)/(1+alpha1).*cg.^(-gamma).*MKT- ones(T,1)- ones(T,1)*alpha0;

% get sample Variance-Covariance matrix
varcov_Moments_num = getVarCovMoments(euler1_func,euler2_func,sdfpar_gmm,T);
varcov_Moments_cond_endow_num = getVarCovMoments(euler1_cond_endow_func,euler2_cond_endow_func,sdfpar_cond_endow_gmm,T);
varcov_Moments_cond_prod2_num = getVarCovMoments(euler1_cond_prod2_func,euler2_cond_prod2_func,sdfpar_cond_prod2_gmm,T);
varcov_Moments_cond_euler_num = getVarCovMoments(euler1_cond_euler_func,euler2_cond_euler_func,sdfpar_cond_euler_gmm,T);
varcov_Moments_cond_PJfix_num = getVarCovMoments(euler1_cond_PJfix_func,euler2_cond_PJfix_func,sdfpar_cond_PJfix_gmm,T);

% Optimal GMM estimates
W = varcov_Moments_num\eye(size(varcov_Moments_num,1));
W_cond_endow = varcov_Moments_cond_endow_num\eye(size(varcov_Moments_cond_endow_num,1));
W_cond_prod2 = varcov_Moments_cond_prod2_num\eye(size(varcov_Moments_cond_prod2_num,1));
W_cond_euler = varcov_Moments_cond_euler_num\eye(size(varcov_Moments_cond_euler_num,1));
W_cond_PJfix = varcov_Moments_cond_PJfix_num\eye(size(varcov_Moments_cond_PJfix_num,1));
[sdfpar_gmm_opt,fval_gmm_opt,exitflag_gmm_opt] = fminsearch(@gmm_2_func,sdfpar_gmm,options,y,X,Z,W);  % Call minimizer
[sdfpar_cond_endow_gmm_opt,fval_cond_endow_gmm_opt,exitflag_cond_endow_gmm_opt] = fminsearch(@gmm_cond_endow_func,b,options,y,X,Z,W_cond_endow,freq,nu_bar,lambda);      % Endowment
[sdfpar_cond_prod2_gmm_opt,fval_cond_prod2_gmm_opt,exitflag_cond_prod2_gmm_opt] = fminsearch(@gmm_cond_prod2_func,b,options,y,X,Z,W_cond_prod2,freq,alpha,nu,lambda,mu_bar,sigma_bar,sigma);  % Production (2)
[sdfpar_cond_euler_gmm_opt,~,~] = fminsearch(@gmm_cond_euler_func,[0;0],options,y,X,Z,W_cond_euler,freq,gamma,rho);  % (33)
[sdfpar_cond_PJfix_gmm_opt,~,~] = fminsearch(@gmm_cond_PJfix_1_func,[0.1;1],options,y,X,Z,W_cond_PJfix,freq,rho,alpha1);  % (34)

sdfpar_gmm_iter = sdfpar_gmm;
sdfpar_cond_endow_gmm_iter = sdfpar_cond_endow_gmm;
sdfpar_cond_prod2_gmm_iter = sdfpar_cond_prod2_gmm;
sdfpar_cond_euler_gmm_iter = sdfpar_cond_euler_gmm;
sdfpar_cond_PJfix_gmm_iter = sdfpar_cond_PJfix_gmm;

% Iterated GMM estimates
while norm([sdfpar_gmm_iter-sdfpar_gmm_opt,sdfpar_cond_endow_gmm_iter-sdfpar_cond_endow_gmm_opt,sdfpar_cond_prod2_gmm_iter-sdfpar_cond_prod2_gmm_opt,sdfpar_cond_euler_gmm_iter-sdfpar_cond_euler_gmm_opt,sdfpar_cond_PJfix_gmm_iter-sdfpar_cond_PJfix_gmm_opt])>1e-04 
    sdfpar_gmm_iter = sdfpar_gmm_opt;
    sdfpar_cond_endow_gmm_iter = sdfpar_cond_endow_gmm_opt;
    sdfpar_cond_prod2_gmm_iter = sdfpar_cond_prod2_gmm_opt;
    sdfpar_cond_euler_gmm_iter = sdfpar_cond_euler_gmm_opt;
    sdfpar_cond_PJfix_gmm_iter = sdfpar_cond_PJfix_gmm_opt;
    varcov_Moments_num = getVarCovMoments(euler1_func,euler2_func,sdfpar_gmm_iter,T);
    varcov_Moments_cond_endow_num = getVarCovMoments(euler1_cond_endow_func,euler2_cond_endow_func,sdfpar_cond_endow_gmm_iter,T);
    varcov_Moments_cond_prod2_num = getVarCovMoments(euler1_cond_prod2_func,euler2_cond_prod2_func,sdfpar_cond_prod2_gmm_iter,T);
    varcov_Moments_cond_euler_num = getVarCovMoments(euler1_cond_euler_func,euler2_cond_euler_func,sdfpar_cond_euler_gmm_iter,T);
    varcov_Moments_cond_PJfix_num = getVarCovMoments(euler1_cond_PJfix_func,euler2_cond_PJfix_func,sdfpar_cond_PJfix_gmm_iter,T);
    W = varcov_Moments_num\eye(size(varcov_Moments_num,1));
    W_cond_endow = varcov_Moments_cond_endow_num\eye(size(varcov_Moments_cond_endow_num,1));
    W_cond_prod2 = varcov_Moments_cond_prod2_num\eye(size(varcov_Moments_cond_prod2_num,1));
    W_cond_euler = varcov_Moments_cond_euler_num\eye(size(varcov_Moments_cond_euler_num,1));
    W_cond_PJfix = varcov_Moments_cond_PJfix_num\eye(size(varcov_Moments_cond_PJfix_num,1));
    [sdfpar_gmm_opt,fval_gmm_opt,exitflag_gmm_opt] = fminsearch(@gmm_2_func,sdfpar_gmm_iter,options,y,X,Z,varcov_Moments_num\eye(size(varcov_Moments_num,1)));  % Call minimizer
    [sdfpar_cond_endow_gmm_opt,fval_cond_endow_gmm_opt,exitflag_cond_endow_gmm_opt] = fminsearch(@gmm_cond_endow_func,sdfpar_cond_endow_gmm_iter,options,y,X,Z,W_cond_endow,freq,nu_bar,lambda);      % Endowment
    [sdfpar_cond_prod2_gmm_opt,fval_cond_prod2_gmm_opt,exitflag_cond_prod2_gmm_opt] = fminsearch(@gmm_cond_prod2_func,sdfpar_cond_prod2_gmm_iter,options,y,X,Z,W_cond_prod2,freq,alpha,nu,lambda,mu_bar,sigma_bar,sigma);  % Production (2)
    [sdfpar_cond_euler_gmm_opt,~,~] = fminsearch(@gmm_cond_euler_func,[0;0],options,y,X,Z,W_cond_euler,freq,gamma,rho);  % (33)
    [sdfpar_cond_PJfix_gmm_opt,~,~] = fminsearch(@gmm_cond_PJfix_1_func,[0.1;1],options,y,X,Z,W_cond_PJfix,freq,rho,alpha1);  % (34)
end

% get asymptotic variance-covariance matrix
varcov_GMM_num = getVarCov(euler1_func,euler2_func,sdfpar_gmm,T);
varcov_GMM_opt_num = getVarCov(euler1_func,euler2_func,sdfpar_gmm_opt,T);
varcov_GMM_cond_endow_num = getVarCov(euler1_cond_endow_func,euler2_cond_endow_func,sdfpar_cond_endow_gmm,T);
varcov_GMM_cond_endow_opt_num = getVarCov(euler1_cond_endow_func,euler2_cond_endow_func,sdfpar_cond_endow_gmm_opt,T);
varcov_GMM_cond_prod2_num = getVarCov(euler1_cond_prod2_func,euler2_cond_prod2_func,sdfpar_cond_prod2_gmm,T);
varcov_GMM_cond_prod2_opt_num = getVarCov(euler1_cond_prod2_func,euler2_cond_prod2_func,sdfpar_cond_prod2_gmm_opt,T);
varcov_GMM_cond_euler_num = getVarCov(euler1_cond_euler_func,euler2_cond_euler_func,sdfpar_cond_euler_gmm,T);
varcov_GMM_cond_euler_opt_num = getVarCov(euler1_cond_euler_func,euler2_cond_euler_func,sdfpar_cond_euler_gmm_opt,T);
varcov_GMM_cond_PJfix_num = getVarCov(euler1_cond_PJfix_func,euler2_cond_PJfix_func,sdfpar_cond_PJfix_gmm,T);
varcov_GMM_cond_PJfix_opt_num = getVarCov(euler1_cond_PJfix_func,euler2_cond_PJfix_func,sdfpar_cond_PJfix_gmm_opt,T);

% asymptotic standard errors
se_gmm(1,:) = sqrt(varcov_GMM_num(1,1));
se_gmm(2,:) = sqrt(varcov_GMM_num(2,2));
se_gmm_opt(1,:) = sqrt(varcov_GMM_opt_num(1,1));
se_gmm_opt(2,:) = sqrt(varcov_GMM_opt_num(2,2));
se_cond_endow_gmm(1,:) = sqrt(varcov_GMM_cond_endow_num(1,1));
se_cond_endow_gmm(2,:) = sqrt(varcov_GMM_cond_endow_num(2,2));
se_cond_endow_gmm_opt(1,:) = sqrt(varcov_GMM_cond_endow_opt_num(1,1));
se_cond_endow_gmm_opt(2,:) = sqrt(varcov_GMM_cond_endow_opt_num(2,2));
se_cond_prod2_gmm(1,:) = sqrt(varcov_GMM_cond_prod2_num(1,1));
se_cond_prod2_gmm(2,:) = sqrt(varcov_GMM_cond_prod2_num(2,2));
se_cond_prod2_gmm_opt(1,:) = sqrt(varcov_GMM_cond_prod2_opt_num(1,1));
se_cond_prod2_gmm_opt(2,:) = sqrt(varcov_GMM_cond_prod2_opt_num(2,2));
se_cond_euler_gmm(1,:) = sqrt(varcov_GMM_cond_euler_num(1,1));
se_cond_euler_gmm(2,:) = sqrt(varcov_GMM_cond_euler_num(2,2));
se_cond_euler_gmm_opt(1,:) = sqrt(varcov_GMM_cond_euler_opt_num(1,1));
se_cond_euler_gmm_opt(2,:) = sqrt(varcov_GMM_cond_euler_opt_num(2,2));
se_cond_PJfix_gmm(1,:) = sqrt(varcov_GMM_cond_PJfix_num(1,1));
se_cond_PJfix_gmm(2,:) = sqrt(varcov_GMM_cond_PJfix_num(2,2));
se_cond_PJfix_gmm_opt(1,:) = sqrt(varcov_GMM_cond_PJfix_opt_num(1,1));
se_cond_PJfix_gmm_opt(2,:) = sqrt(varcov_GMM_cond_PJfix_opt_num(2,2));

% square root of the average squared EE Error
rmse=sqrt(0.5*(mean(euler1_func(sdfpar_gmm_opt(1),sdfpar_gmm_opt(2))).^2 + mean(euler2_func(sdfpar_gmm_opt(1),sdfpar_gmm_opt(2))).^2))/freq;                                              % RMSE
rmse_cond_endow=sqrt(0.5*(mean(euler1_cond_endow_func(sdfpar_cond_endow_gmm_opt(1),sdfpar_cond_endow_gmm_opt(2))).^2 + mean(euler2_cond_endow_func(sdfpar_cond_endow_gmm_opt(1),sdfpar_cond_endow_gmm_opt(2))).^2))/freq;                                              % RMSE
rmse_cond_prod2=sqrt(0.5*(mean(euler1_cond_prod2_func(sdfpar_cond_prod2_gmm_opt(1),sdfpar_cond_prod2_gmm_opt(2))).^2 + mean(euler2_cond_prod2_func(sdfpar_cond_prod2_gmm_opt(1),sdfpar_cond_prod2_gmm_opt(2))).^2))/freq;                                              % RMSE
rmse_cond_euler=sqrt(0.5*(mean(euler1_cond_euler_func(sdfpar_cond_euler_gmm_opt(1),sdfpar_cond_euler_gmm_opt(2))).^2 + mean(euler2_cond_euler_func(sdfpar_cond_euler_gmm_opt(1),sdfpar_cond_euler_gmm_opt(2))).^2))/freq;                                              % RMSE
rmse_cond_PJfix=sqrt(0.5*(mean(euler1_cond_PJfix_func(sdfpar_cond_PJfix_gmm_opt(1),sdfpar_cond_PJfix_gmm_opt(2))).^2 + mean(euler2_cond_PJfix_func(sdfpar_cond_PJfix_gmm_opt(1),sdfpar_cond_PJfix_gmm_opt(2))).^2))/freq;                                              % RMSE
rmsr=sqrt(0.5*((mean(RF) -1)^2 + (mean(MKT)-1)^2))/freq;

% summary
ctrystr
fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{%4.1f}{%4.1f} $&$\\ \bunderset{[%1.2f]}{%1.1f}$\n',sdfpar_gmm_opt(1)/se_gmm_opt(1),sdfpar_gmm_opt(1),sdfpar_gmm_opt(2)/se_gmm_opt(2),sdfpar_gmm_opt(2),rmse/rmsr,rmse*100)
fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{%4.1f}{%4.1f} $&$\\ \bunderset{[%1.2f]}{%1.1f}$\n',sdfpar_cond_endow_gmm_opt(1)/se_cond_endow_gmm_opt(1),sdfpar_cond_endow_gmm_opt(1),sdfpar_cond_endow_gmm_opt(2)/se_cond_endow_gmm_opt(2),sdfpar_cond_endow_gmm_opt(2),rmse_cond_endow/rmsr,rmse_cond_endow*100)
fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{%4.1f}{%4.1f} $&$\\ \bunderset{[%1.2f]}{%1.1f}$\n',sdfpar_cond_prod2_gmm_opt(1)/se_cond_prod2_gmm_opt(1),sdfpar_cond_prod2_gmm_opt(1),sdfpar_cond_prod2_gmm_opt(2)/se_cond_prod2_gmm_opt(2),sdfpar_cond_prod2_gmm_opt(2),rmse_cond_prod2/rmsr,rmse_cond_prod2*100)
fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{[%1.2f]}{%1.1f}$\n',sdfpar_cond_euler_gmm_opt(1)/se_cond_euler_gmm_opt(1),sdfpar_cond_euler_gmm_opt(1),sdfpar_cond_euler_gmm_opt(2)/se_cond_euler_gmm_opt(2),sdfpar_cond_euler_gmm_opt(2),rmse_cond_euler/rmsr,rmse_cond_euler*100)
fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $&$\\ \bunderset{%4.1f}{%4.1f} $&$\\ \bunderset{[%1.2f]}{%1.1f}$\\\\\n',sdfpar_cond_PJfix_gmm_opt(1)/se_cond_PJfix_gmm_opt(1),sdfpar_cond_PJfix_gmm_opt(1),sdfpar_cond_PJfix_gmm_opt(2)/se_cond_PJfix_gmm_opt(2),sdfpar_cond_PJfix_gmm_opt(2),rmse_cond_PJfix/rmsr,rmse_cond_PJfix*100)

%-------------------------------------------------------------------------%
% Table A.2
%-------------------------------------------------------------------------%

if data==2 && sample == 2 && ctry == 15
    
    % 8 assets (riskless, risky, FF)
    y = [ones(T,1) ones(T,1) ones(T,1) ones(T,1) ones(T,1) ones(T,1) ones(T,1) ones(T,1)];   % matrix containing the population mean of moment conditions
    X = [cg MKT MKTX FF];
    Z = ones(T,1);                % vector of instruments
    [b_8_gmm,~,~] = fminsearch(@gmm_8_func,b(1:2),options,y,X,Z,eye(8));  % Call minimizer
    [b_8_gmm_cond_endow,fval_8_gmm,exitflag_8_gmm] = fminsearch(@gmm_8_cond_endow_func,[.95;1],options,y,X,Z,eye(8),freq,nu_bar,lambda);  % Call minimizer
    [b_8_gmm_cond_euler,~,~] = fminsearch(@gmm_8_cond_euler_func,[0;0;0;0;0;0;0;0],options,y,X,Z,eye(8),freq,gamma,rho);  % (33)
    [b_8_gmm_cond_euler_2,~,~] = fminsearch(@gmm_8_cond_euler_2_func,[0;0;1;2;0;0;0;0],options,y,X,Z,eye(8),freq,gamma,rho);  % (33)
    
    %-------------------------------------------------------------------------%
    % GMM, disaster risk estimates (as reported in Section 4.5)
    %-------------------------------------------------------------------------%
    alpha_1 = b_8_gmm_cond_euler_2(1);
    alpha_2 = b_8_gmm_cond_euler_2(2);
%     log((1+alpha_2)/(1+alpha_1))
    [nu_bar_hat,fval,exitflag]=fsolve(@(nu_bar) (1-exp((1-gamma)*nu_bar))/(1-exp(-nu_bar*gamma))-log(1+alpha_2)/log(1+alpha_1), -0.1, options);
    lambda_hat = log(1+alpha_1)/(1-exp(-nu_bar*gamma))/freq;
    [nu_bar_hat,lambda_hat]
    
    euler1_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*RF-ones(T,1);
    euler2_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*MKT-ones(T,1);
    euler3_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,1)-ones(T,1);
    euler4_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,2)-ones(T,1);
    euler5_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,3)-ones(T,1);
    euler6_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,4)-ones(T,1);
    euler7_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,5)-ones(T,1);
    euler8_8_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,6)-ones(T,1);
    
    euler1_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*RF-ones(T,1)*exp((1-exp(-gamma*nu_bar))*lambda*freq);
    euler2_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*MKT-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler3_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,1)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler4_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,2)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler5_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,3)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler6_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,4)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler7_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,5)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    euler8_8_cond_endow_func = @(beta,gamma) sdf_fitted_func(beta,gamma).*FF(:,6)-ones(T,1)*exp((1-exp((1-gamma)*nu_bar))*lambda*freq);
    
    euler1_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*RF-ones(T,1)-a1;
    euler2_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*MKT-ones(T,1)-a2;
    euler3_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,1)-ones(T,1)-a3;
    euler4_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,2)-ones(T,1)-a4;
    euler5_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,3)-ones(T,1)-a5;
    euler6_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,4)-ones(T,1)-a6;
    euler7_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,5)-ones(T,1)-a7;
    euler8_8_cond_euler_func = @(a1,a2,a3,a4,a5,a6,a7,a8) exp(-rho*freq)*cg.^(-gamma).*FF(:,6)-ones(T,1)-a8;
    
    % Fama/French Research Portfolios
    % Size and Book-to-Market Portfolios
    % [Big Value, Big Neutral, Big Growth, Small Value, Small Neutral, Small Growth]
    euler1_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*RF-ones(T,1)-a1;
    euler2_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*MKT-ones(T,1)-a2;
    euler3_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,1)-ones(T,1)-a5;
    euler4_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,2)-ones(T,1)-a5;
    euler5_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,3)-ones(T,1)-a6;
    euler6_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,4)-ones(T,1)-a5;
    euler7_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,5)-ones(T,1)-a7;
    euler8_8_cond_euler_2_func = @(a1,a2,beta,gamma,a5,a6,a7,a8) beta*cg.^(-gamma).*FF(:,6)-ones(T,1)-a8;
    
    
    rmse_8=sqrt(1/8*(mean(euler1_8_func(b_8_gmm(1),b_8_gmm(2))).^2 + mean(euler2_8_func(b_8_gmm(1),b_8_gmm(2))).^2 + mean(euler3_8_func(b_8_gmm(1),b_8_gmm(2))).^2+ mean(euler4_8_func(b_8_gmm(1),b_8_gmm(2))).^2+ mean(euler5_8_func(b_8_gmm(1),b_8_gmm(2))).^2+ mean(euler6_8_func(b_8_gmm(1),b_8_gmm(2))).^2+ mean(euler7_8_func(b_8_gmm(1),b_8_gmm(2))).^2+ mean(euler8_8_func(b_8_gmm(1),b_8_gmm(2))).^2))/freq;
    rmse_8_cond_endow=sqrt(1/8*(mean(euler1_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2 + mean(euler2_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2 + mean(euler3_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2+ mean(euler4_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2+ mean(euler5_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2+ mean(euler6_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2+ mean(euler7_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2+ mean(euler8_8_cond_endow_func(b_8_gmm_cond_endow(1),b_8_gmm_cond_endow(2))).^2))/freq;
    rmse_8_cond_euler=sqrt(1/8*(mean(euler1_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2 + mean(euler2_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2 + mean(euler3_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2+ mean(euler4_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2+ mean(euler5_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2+ mean(euler6_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2+ mean(euler7_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2+ mean(euler8_8_cond_euler_func(b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8))).^2))/freq;
    rmse_8_cond_euler_2=sqrt(1/8*(mean(euler1_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2 + mean(euler2_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2 + mean(euler3_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2+ mean(euler4_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2+ mean(euler5_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2+ mean(euler6_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2+ mean(euler7_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2+ mean(euler8_8_cond_euler_2_func(b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8))).^2))/freq;
    rmsr_8=sqrt(1/8*((mean(RF) -1)^2 + (mean(MKT)-1)^2+(mean(FF(:,1))-1)^2+(mean(FF(:,2))-1)^2+(mean(FF(:,3))-1)^2+(mean(FF(:,4))-1)^2+(mean(FF(:,5))-1)^2+(mean(FF(:,6))-1)^2))/freq;
    
    varcov_GMM_8_num = getVarCovMoments_8_GMM(euler1_8_func,euler2_8_func,euler3_8_func,euler4_8_func,euler5_8_func,euler6_8_func,euler7_8_func,euler8_8_func,b_8_gmm,T);
    varcov_Moments_8_cond_euler_num = getVarCovMoments_8(euler1_8_cond_euler_func,euler2_8_cond_euler_func,euler3_8_cond_euler_func,euler4_8_cond_euler_func,euler5_8_cond_euler_func,euler6_8_cond_euler_func,euler7_8_cond_euler_func,euler8_8_cond_euler_func,b_8_gmm_cond_euler,T);
    varcov_Moments_8_cond_euler_2_num = getVarCovMoments_8(euler1_8_cond_euler_2_func,euler2_8_cond_euler_2_func,euler3_8_cond_euler_2_func,euler4_8_cond_euler_2_func,euler5_8_cond_euler_2_func,euler6_8_cond_euler_2_func,euler7_8_cond_euler_2_func,euler8_8_cond_euler_2_func,b_8_gmm_cond_euler_2,T);
    
    se_8_cond_euler_gmm(1,:) = sqrt(varcov_Moments_8_cond_euler_num(1,1));
    se_8_cond_euler_gmm(2,:) = sqrt(varcov_Moments_8_cond_euler_num(2,2));
    se_8_cond_euler_gmm(3,:) = sqrt(varcov_Moments_8_cond_euler_num(3,3));
    se_8_cond_euler_gmm(4,:) = sqrt(varcov_Moments_8_cond_euler_num(4,4));
    se_8_cond_euler_gmm(5,:) = sqrt(varcov_Moments_8_cond_euler_num(5,5));
    se_8_cond_euler_gmm(6,:) = sqrt(varcov_Moments_8_cond_euler_num(6,6));
    se_8_cond_euler_gmm(7,:) = sqrt(varcov_Moments_8_cond_euler_num(7,7));
    se_8_cond_euler_gmm(8,:) = sqrt(varcov_Moments_8_cond_euler_num(8,8));
    
    se_8_cond_euler_2_gmm(1,:) = sqrt(varcov_Moments_8_cond_euler_2_num(1,1));
    se_8_cond_euler_2_gmm(2,:) = sqrt(varcov_Moments_8_cond_euler_2_num(2,2));
    se_8_cond_euler_2_gmm(3,:) = sqrt(varcov_Moments_8_cond_euler_2_num(3,3));
    se_8_cond_euler_2_gmm(4,:) = sqrt(varcov_Moments_8_cond_euler_2_num(4,4));
    se_8_cond_euler_2_gmm(5,:) = sqrt(varcov_Moments_8_cond_euler_2_num(5,5));
    se_8_cond_euler_2_gmm(6,:) = sqrt(varcov_Moments_8_cond_euler_2_num(6,6));
    se_8_cond_euler_2_gmm(7,:) = sqrt(varcov_Moments_8_cond_euler_2_num(7,7));
    se_8_cond_euler_2_gmm(8,:) = sqrt(varcov_Moments_8_cond_euler_2_num(8,8));
    
    % summary
    ctrystr
    fprintf('& $\\ \bunderset{}{%4.3f} $& $\\ \bunderset{}{%4.1f}\\ $& $\\ $& $\\ $& $\\ $& $\\ $& $\\ $& $\\ $& $\\ $& $\\ $& $\\ \bunderset{[%1.2f]}{%1.1f} $\\\\\n',b_8_gmm(1),b_8_gmm(2),rmse_8/rmsr_8,rmse_8*100)
    fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{[%1.2f]}{%1.1f} $\\\\\n',b_8_gmm_cond_euler(1)./se_8_cond_euler_gmm(1),b_8_gmm_cond_euler(1),b_8_gmm_cond_euler(2)./se_8_cond_euler_gmm(2),b_8_gmm_cond_euler(2),b_8_gmm_cond_euler(3)./se_8_cond_euler_gmm(3),b_8_gmm_cond_euler(3),b_8_gmm_cond_euler(4)./se_8_cond_euler_gmm(4),b_8_gmm_cond_euler(4),b_8_gmm_cond_euler(5)./se_8_cond_euler_gmm(5),b_8_gmm_cond_euler(5),b_8_gmm_cond_euler(6)./se_8_cond_euler_gmm(6),b_8_gmm_cond_euler(6),b_8_gmm_cond_euler(7)./se_8_cond_euler_gmm(7),b_8_gmm_cond_euler(7),b_8_gmm_cond_euler(8)./se_8_cond_euler_gmm(8),b_8_gmm_cond_euler(8),rmse_8_cond_euler/rmsr_8,rmse_8_cond_euler*100)
    fprintf('& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.1f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{%4.1f}{%4.3f} $& $\\ \bunderset{[%1.2f]}{%1.1f} $\\\\\n',b_8_gmm_cond_euler_2(3)./se_8_cond_euler_2_gmm(3),b_8_gmm_cond_euler_2(3),b_8_gmm_cond_euler_2(4)./se_8_cond_euler_2_gmm(4),b_8_gmm_cond_euler_2(4),b_8_gmm_cond_euler_2(1)./se_8_cond_euler_2_gmm(1),b_8_gmm_cond_euler_2(1),b_8_gmm_cond_euler_2(2)./se_8_cond_euler_2_gmm(2),b_8_gmm_cond_euler_2(2),b_8_gmm_cond_euler_2(5)./se_8_cond_euler_2_gmm(5),b_8_gmm_cond_euler_2(5),b_8_gmm_cond_euler_2(6)./se_8_cond_euler_2_gmm(6),b_8_gmm_cond_euler_2(6),b_8_gmm_cond_euler_2(7)./se_8_cond_euler_2_gmm(7),b_8_gmm_cond_euler_2(7),b_8_gmm_cond_euler_2(8)./se_8_cond_euler_2_gmm(8),b_8_gmm_cond_euler_2(8),rmse_8_cond_euler_2/rmsr_8,rmse_8_cond_euler_2*100)
end

%% Auxiliary functions

function varcov_Estimate = getVarCovMoments(f1,f2,param,T)
% Get sample covariance
%   Computes covariance for given function and parameters
h = [f1(param(1),param(2)) f2(param(1),param(2))]';
varcov_Estimate = 1/T*(h*h');
end

function varcov_GMM = getVarCov(f1,f2,param,T)
% Get asymptotic covariance
%   Computes asymptotic covariance for given function and parameters
h = [f1(param(1),param(2)) f2(param(1),param(2))]';
h_11_num = fjac(f1,1,param(1),param(2));
h_12_num = fjac(f1,2,param(1),param(2));
h_21_num = fjac(f2,1,param(1),param(2));
h_22_num = fjac(f2,2,param(1),param(2));
jac_num = zeros(2,2,T);
for i=1:T
    jac_num(:,:,i) = [h_11_num(i) h_12_num(i); h_21_num(i) h_22_num(i)];
end
varcov_GMM = (mean(jac_num,3)'*((1/T*(h*h'))\eye(2))*mean(jac_num,3))\eye(2);
end

function [f,b]=gmm_2_func(b,y,X,Z,W)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
R = [RF MKT];
cg = X(:,1);
e = repmat(b(1)*cg.^(-b(2)),1,neq).*R - y;
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_cond_endow_func(b,y,X,Z,W,freq,nu_bar,lambda)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
gamma = b(2);
e = repmat(imrs,1,neq).*R - y*[exp(-(exp(-gamma*nu_bar)-1)*lambda*freq),0;0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b,delta]=gmm_cond_prod2_func(b,y,X,Z,W,freq,alpha,nu,lambda,mu_bar,sigma_bar,sigma)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y*[exp((1-exp(-alpha*b(2)*nu))*lambda*freq),0;0,1];
m = reshape(Z'*e/T,north,1);
rho=-log(b(1))/freq;
delta = ((exp((1-alpha*b(2))*nu)-1)*lambda-b(2)*mu_bar+0.5*(b(2)*(1+b(2))*sigma_bar^2-alpha*b(2)*(1-alpha*b(2))*sigma^2)-rho)/(1-alpha*b(2));
% fprintf('implied delta = %1.4f\n',delta);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_cond_euler_func(b,y,X,Z,W,freq,gamma,rho)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
R = [RF MKT];
cg = X(:,1);
imrs = exp(-rho*freq)*cg.^(-gamma);
e = repmat(imrs,1,neq).*R - y*[1+b(1),0;0,1+b(2)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_cond_PJfix_1_func(b,y,X,Z,W,freq,rho,alpha1)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
R = [RF (exp(-rho*freq)/(1+alpha1)).*MKT];
cg = X(:,1);
imrs = cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y*[1/(exp(-rho*freq)/(1+alpha1)),0;0,1+b(1)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_8_func(b,y,X,Z,W)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2) X(:,4:end)];
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
e = repmat(imrs,1,neq).*R - y;
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_8_cond_endow_func(b,y,X,Z,W,freq,nu_bar,lambda)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
R = [X(:,2)-X(:,3) X(:,2) X(:,4:end)];% R = X(:,2);
cg = X(:,1);
imrs = b(1)*cg.^(-b(2));
gamma = b(2);
e = repmat(imrs,1,neq).*R - y*[exp(-(exp(-gamma*nu_bar)-1)*exp(lambda)*freq),0,0,0,0,0,0,0;0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0,0,0,0,0,0;0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0,0,0,0,0;0,0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0,0,0,0;0,0,0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0,0,0;0,0,0,0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0,0;0,0,0,0,0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq),0;0,0,0,0,0,0,0,exp(-(exp((1-gamma)*nu_bar)-1)*lambda*freq)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_8_cond_euler_func(b,y,X,Z,W,freq,gamma,rho)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
FF=X(:,4:end);
R = [RF MKT FF];% R = X(:,2);
cg = X(:,1);
imrs = exp(-rho*freq)*cg.^(-gamma);
e = repmat(imrs,1,neq).*R - y*[1+b(1),0,0,0,0,0,0,0;
                               0,1+b(2),0,0,0,0,0,0;...
                               0,0,1+b(3),0,0,0,0,0;...
                               0,0,0,1+b(4),0,0,0,0;...
                               0,0,0,0,1+b(5),0,0,0;...
                               0,0,0,0,0,1+b(6),0,0;...
                               0,0,0,0,0,0,1+b(7),0;...
                               0,0,0,0,0,0,0,1+b(8)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function [f,b]=gmm_8_cond_euler_2_func(b,y,X,Z,W,~,~,~)
[T, neq] = size(y);
k = size(Z,2);
north = neq*k;
RF = X(:,2)-X(:,3);
MKT = X(:,2);
FF=X(:,4:end);
R = [RF MKT FF];% R = X(:,2);
cg = X(:,1);
imrs = b(3)*cg.^(-b(4));
% Fama/French Research Portfolios
% Size and Book-to-Market Portfolios 
% [Big Value, Big Neutral, Big Growth, Small Value, Small Neutral, Small Growth]                            
e = repmat(imrs,1,neq).*R - y*[1+b(1),0,0,0,0,0,0,0;
                               0,1+b(2),0,0,0,0,0,0;...
                               0,0,1+b(5),0,0,0,0,0;...
                               0,0,0,1+b(5),0,0,0,0;...
                               0,0,0,0,1+b(6),0,0,0;...
                               0,0,0,0,0,1+b(5),0,0;...
                               0,0,0,0,0,0,1+b(7),0;...
                               0,0,0,0,0,0,0,1+b(8)];
m = reshape(Z'*e/T,north,1);
f=m'*W*m;       % setup objective function
end

function varcov_Estimate = getVarCovMoments_8_GMM(f1,f2,f3,f4,f5,f6,f7,f8,param,T)
% Get sample covariance
%   Computes covariance for given function and parameters
h = [f1(param(1),param(2)) f2(param(1),param(2)) f3(param(1),param(2)) f4(param(1),param(2)) f5(param(1),param(2)) f6(param(1),param(2)) f7(param(1),param(2)) f8(param(1),param(2))]';
varcov_Estimate = 1/T*(h*h');
end

function varcov_Estimate = getVarCovMoments_8(f1,f2,f3,f4,f5,f6,f7,f8,param,T)
% Get sample covariance
%   Computes covariance for given function and parameters
h = [f1(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f2(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f3(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f4(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f5(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f6(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f7(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8)) f8(param(1),param(2),param(3),param(4),param(5),param(6),param(7),param(8))]';
varcov_Estimate = 1/T*(h*h');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
