%% =======================================================================%
%  Peso Problems in the Estimation of the C-CAPM                          %
%  Simulation of the rare disaster and long-run risk (LRR) models         %    
%  Quantitative Economics (forthcoming)                                   %
%
%  (0) Housekeeping                                                       %  
%  (1) Sampling                                                           %  
%  (2) Estimation                                                         %
%  (3) Illustration                                                       %
% ----------------------------------------------------------------------- %
%  NOTES: 
% ----------------------------------------------------------------------- %
%  Main.m requires the following complementary codes available in the 
%  folder:
%
%  1. main_sampling.m : Simulates paths
%
%  2. main_stochastics.m : New realizations from distributions
%
%  3. main_pricing.m : Asset pricing
%
%  4. main_estimation.m : Estimation
%
%  5. main_summary.m : Histograms
%
%  implemented using Matlab R2021a with toolboxes stats, optim
%  additional files for LRR model (see main_LRR.m)

% The model is selected by choosing 'solution' to take the value either
% 
% 1: Lucas' endowment economy with rare events (Section 3.1)
% 3: A mimicking an endowment economy (Section 3.2)
% 4: LRR model (Bansal-Yaron)
% ----------------------------------------------------------------------- %
% (c) Juan Carlos Parra-Alvarez, Olaf Posch and Andreas Schrimpf            
% Date July, 2021.
% ----------------------------------------------------------------------- %

%% =======================================================================%
% (0) Housekeeping                                                        %
%-------------------------------------------------------------------------%
clear
clc
close all

%-------------------------------------------------------------------------%
% Simulate data                                                           %
%-------------------------------------------------------------------------%
% specify date for simulated data, a new folder will be created
mypath = cd;
startdate = date;

% specify if new realizations should be drawn (for rare disaster models)
% 0 use realizations from the draw 'startdate'
% 1 new realizations are generated
stochastics = 1;

%-------------------------------------------------------------------------%
% Save output                                                             %
%-------------------------------------------------------------------------%
save_output = 1;

%-------------------------------------------------------------------------%
% Summary                                                                 %
%-------------------------------------------------------------------------%
figures  = 1;

%-------------------------------------------------------------------------%
% Setting up simulaton study design
%-------------------------------------------------------------------------%

% oberservation period in years
nPeriods 	= 50;             
% sampling precision, e.g., prec = 1/10 
% (should be a multiple of the observation frequency)
prec = 1/12; 
index = zeros(nPeriods/prec,1);
index((1:nPeriods)./prec)=1;
% observation frequency, e.g., 1/12 is monthly
freq = 1/4; 
if (prec>freq) 
    disp('error: observation frequency higher than sampling precision'); 
end
index_obs = zeros(nPeriods/prec,1);
index_obs((1:nPeriods/freq)./(prec/freq))=1;

%-------------------------------------------------------------------------%
% determine the number of samples to be drawn
%-------------------------------------------------------------------------%
loops = 5000;
results_mat = NaN(loops,36);

%% =======================================================================%
% (1) Sampling
%-------------------------------------------------------------------------%
% specify the rare disaster/long-run risk model 
%-------------------------------------------------------------------------%
% 1: Lucas' endowment economy with rare events (Section 3.1)
% 3: A mimicking an endowment economy (Section 3.2)
% 4: LRR model (Bansal-Yaron)

solution = 1;

%-------------------------------------------------------------------------%
% choose quiet for a sample without rare disasters 
%-------------------------------------------------------------------------%
quiet = 1;

if solution==1
    % Lucas' fruit tree economy
    % parameterization (1) 
    rho        = .03;     % rate of time preference
    gamma      = 4;       % coef. of relative risk aversion
    mu_bar     = .01;     % consumption growth
    sigma_bar  = .005;    % consumption noise
    nu_bar     = -0.4;    % size of consumption disaster
    lambda     = 0.017;   % consumption disaster probability
    kappa      = -0.3;    % size of government default
    q          = 0.5;     % default probability
%     % parameterization (2) 
%     rho        = .03;   % rate of time preference
%     gamma      = 4;     % coef. of relative risk aversion
%     mu_bar     = .01;   % consumption growth
%     sigma_bar  = .005;  % consumption noise
%     nu_bar     = 0;     % size of consumption disaster
%     lambda     = 0;     % consumption disaster probability
%     kappa      = 0;     % size of government default
%     q          = 0;     % default probability

    % model remaining parameters (not used in this model)
    alpha      = 0;     
    delta      = 0;
    nu         = 0;     
    sigma      = 0;
    % measurement errors (not used)
    eta = 0;

elseif solution==3
    % production economy
    % parameterization (1)
    gamma      = 4;      % coef. of relative risk aversion
    alpha      = .6;     % output elasticity of capital
    delta      = .025;   % capital depreciation
    mu_bar     = .01;   % productivity growth
    sigma_bar  = .01;    % productivity noise
    nu_bar     = -0.01;   % size of productivity slump
    lambda_bar = 0;    % productivity jump probability
    sigma      = 0.005;  % capital stochastic depreciation
    nu         = -.55;   % size of capital disaster
    lambda     = .017;   % capital disaster probability
      % parameterization (2)
%     gamma      = 4;      % coef. of relative risk aversion
%     alpha      = .6;     % output elasticity of capital
%     delta      = .025;   % capital depreciation
%     mu_bar     = .01;   % productivity growth
%     sigma_bar  = .01;    % productivity noise
%     nu_bar     = 0;   % size of productivity slump
%     lambda_bar = 0;    % productivity jump probability
%     sigma      = 0.005;  % capital stochastic depreciation
%     nu         = -.55;   % size of capital disaster
%     lambda     = .017;   % capital disaster probability

    % implied time preference for explicit solution
    rho = ((exp(-gamma*nu_bar)-1)*lambda_bar+(exp((1-alpha*gamma)*nu)-1)*lambda-gamma*mu_bar+.5*(gamma*(1+gamma)*sigma_bar^2-alpha*gamma*(1-alpha*gamma)*sigma^2)-(1-alpha*gamma)*delta);

    % measurement error (not used)
    eta = 0;
end

% creating initial period for recursive formulation and check restrictions
if (solution == 3)
    nPeriods=(nPeriods+1);
    % check condition for constant-saving-function (Cobb-Douglas)
    if (gamma > 1) && (rho == ((exp(-gamma*nu_bar)-1)*lambda_bar+(exp((1-alpha*gamma)*nu)-1)*lambda-gamma*mu_bar+.5*(gamma*(1+gamma)*sigma_bar^2-alpha*gamma*(1-alpha*gamma)*sigma^2)-(1-alpha*gamma)*delta)) && (solution == 3)
        disp('Checking condition for constant-saving-function (Cobb-Douglas): fulfilled.');
    else
        return
    end
end

if (solution==1) 
    lambda2 = 0;          % no second jump process in the endowment case
end

if (solution ==3) 
    lambda2 = lambda;     % lambda2 is disaster probability in the production case
    lambda = lambda_bar;  % lambda is TFP jump probability in the production case
    kappa = 0;            % no default in the production case
    q = 0;                % no probability of default in the production case
end

if (solution~=4)
    if (stochastics == 1) 
        warning off MATLAB:MKDIR:DirectoryExists 
        mkdir([mypath,'/Data/Draw_',startdate,'/Samples']); 
        disp('New stochastics are drawn.');
    else
        disp(['Using the draw from ',startdate,'.']);
    end
    loopbar = waitbar(0,'Sampling');
    
    % initialize objects for sampling
    outr_xnorm=[]; outr_xnorm2=[]; outr_xnorm3=[]; outr_xpois=[];  
    outr_xpois2=[]; outr_xdefs=[]; outr_xjump=[]; outr_xjump2=[]; 
    outr_xbins=[]; outr_y=[]; outr_k=[]; outr_rsim=[]; outr_rds=[];
    
    % sampling loops
    for loop = 1:loops
        main_sampling;
    end
    % save output
    if save_output 
        savefile=['Data/Draw_' startdate '/Samples/'  'RES_SAMP.mat'];
        save(savefile,'outr_xnorm','outr_xnorm2','outr_xnorm3','outr_xpois','outr_xpois2','outr_xjump','outr_xjump2','outr_xbins','outr_xdefs');
        savefile2=['Data/Draw_' startdate '/Samples/'  'RES_SAMP2.mat'];
        save(savefile2,'outr_y','outr_k','outr_rsim','outr_rds','B_init','Z_init','N_init','N2_init');
    end
    close(loopbar);
    disp(['Sampling:',' done.'])

elseif solution == 4
    quiet = 0;
    if save_output
        warning off MATLAB:MKDIR:DirectoryExists
        mkdir([mypath,'/Data/Draw_',startdate,'/Samples']); 
        run 'LRR/main_LRR(startdate)' 
    end
    % long-run risk model
    % parameterization (1)
    simversion ='v1';
    load(['Data/Draw_',startdate,'/Samples/LRR_Sim_',simversion,'.mat']);   
    switch simversion
        case 'v1'
            rho = 0.024;
            gamma = 10;
        case 'v2'
            rho = 0.03;
            gamma = 10;
    end
end

%% =======================================================================%
% (2) Asset pricing and estimation
%-------------------------------------------------------------------------%
if (solution ~= 4)
    load(['Data/Draw_' startdate '/Samples/'  'RES_SAMP.mat']);
    load(['Data/Draw_' startdate '/Samples/'  'RES_SAMP2.mat']);
else
    load(['Data/Draw_',startdate,'/Samples/LRR_Sim_',simversion,'.mat']);   
end

loopbar = waitbar(0,'Estimation');
for loop = 1:loops
    main_pricing
    main_estimation
    waitbar(loop/loops)
end

close(loopbar)
disp(['Estimation:',' done.'])

% write estimation results to file
if save_output
    if (quiet == 1)
        dlmwrite(['Data/Draw_',startdate,'/calibration_',num2str(solution),'_cond.txt'],[mu_bar sigma_bar kappa q nu_bar lambda nu lambda2 sigma quiet gamma alpha delta rho],'delimiter',';');
        dlmwrite(['Data/Draw_',startdate,'/results_mat_',num2str(solution),'_cond.txt'],results_mat,'delimiter',';');
    else
        if solution ~=4
            dlmwrite(['Data/Draw_',startdate,'/calibration_',num2str(solution),'.txt'],[mu_bar sigma_bar kappa q nu_bar lambda nu lambda2 sigma quiet gamma alpha delta rho],'delimiter',';');
            dlmwrite(['Data/Draw_',startdate,'/results_mat_',num2str(solution),'.txt'],results_mat,'delimiter',';');
        else
            dlmwrite(['Data/Draw_',startdate,'/results_mat_',num2str(solution)','_',simversion,'.txt'],results_mat,'delimiter',';');
        end
    end
end

%% =======================================================================%
% (3) Illustrations
%-------------------------------------------------------------------------%

if (figures == 1) 
    % show histograms for illustration
    main_summary
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
